! This file is part of FD4: Four-Dimensional Distributed Dynamic Data structures
!  
! Copyright 2010-2011 Matthias Lieber, ZIH, TU Dresden, Germany
! matthias.lieber@tu-dresden.de
!  
! FD4 is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.
!  
! FD4 is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!  
! You should have received a copy of the GNU General Public License
! along with FD4.  If not, see <http://www.gnu.org/licenses/>.

!--------------------------------------------------------------!
!                                                              !
!            Data related with a neighbor partition            !
!                                                              !
!--------------------------------------------------------------!

#include "fd4.h"

!! Data related with a neighbor partition.
!!
!! @author Matthias Lieber
module fd4_neigh_mod

  use fd4_block_mod, only: fd4_r_k_link
  use fd4_globaldef_mod
  use fd4_mpitype_mod
  use kinds_mod
  
  implicit none
  private
  
  public fd4_neigh, fd4_neigh_deallocate, fd4_mpitype, fd4_r_k_link
  public FD4_NEIGH_SEND, FD4_NEIGH_RECV
  
  integer(i_k), parameter :: FD4_NEIGH_SEND = 1
  integer(i_k), parameter :: FD4_NEIGH_RECV = 2
  
  type fd4_neigh
    ! stuff for constructing ghost communication MPI datatypes
    type(fd4_mpitype) :: ghosttype(2)
    type(fd4_r_k_link) :: minptr(2)
    logical :: changed = .false.                                  !! flag to notify ghostcomm if partitioning has changed
  end type fd4_neigh

contains

#ifdef COMMENT
  !! Allocate all data of the $fd4_neigh
  subroutine fd4_neigh_allocate(n, datatype_count)
    type(fd4_neigh), intent(inout) :: n             !! pointer to $fd4_neigh to allocate
    integer(i_k), intent(in) :: datatype_count      !! number of datatypes to allocate
    
    !TODO: delete this routine?
    
    n%count = 0
  end subroutine fd4_neigh_allocate
#endif

  !! Deallocate all data of the $fd4_neigh
  subroutine fd4_neigh_deallocate(n)
    type(fd4_neigh), intent(inout) :: n          !! pointer to $fd4_neigh to deallocate
    call fd4_mpitype_free(n%ghosttype( FD4_NEIGH_SEND), .true., .false. )
    call fd4_mpitype_free(n%ghosttype( FD4_NEIGH_RECV), .true., .false. )
    NULLIFY( n%minptr(FD4_NEIGH_SEND)%l )
    NULLIFY( n%minptr(FD4_NEIGH_RECV)%l )
  end subroutine fd4_neigh_deallocate
  
end module fd4_neigh_mod
